# -*- coding: utf-8 -*-

try:
  set
except NameError:
  from sets import Set as set

from django import template
#from django.http import Http404
from django.core.paginator import Paginator, InvalidPage
from django.conf import settings
from common.decorators import simple_tag
from paginer import types # ech...

register = template.Library()

DEFAULT_PAGINATION = getattr(settings, 'PAGINER_DEFAULT_PAGINATION', 10)
DEFAULT_WINDOW = getattr(settings, 'PAGINER_DEFAULT_WINDOW', 4)
DEFAULT_ORPHANS = getattr(settings, 'PAGINER_DEFAULT_ORPHANS', 0)

@simple_tag(register, takes_context=True)
def autopaginate(context, id, page=1, ajax=False,
                 paginate_by=DEFAULT_PAGINATION, orphans=DEFAULT_ORPHANS):
  list_type = context['paginer'][id]['type']
  params = context['paginer'][id]['params']
  queryset = types[list_type].queryset(params, context['user'])
  paginator = Paginator(queryset, paginate_by, orphans)
  try:
    page_obj = paginator.page(page)
  except InvalidPage:
    context['invalid_page'] = True
    return u''
  if ajax and types[list_type].perm:
    for object in page_obj.object_list:
      if not types[list_type].perm(context['user'], object, 'view'):
        context['invalid_permissions'] = True
        return u''
  context['paginer'][id]['list'] = page_obj.object_list
  if 'paginator' not in context:
    context['paginator'] = {}
  if 'page_obj' not in context:
    context['page_obj'] = {}
  context['paginator'][id] = paginator
  context['page_obj'][id] = page_obj
  return u''

@register.inclusion_tag('pagination.html', takes_context=True)
def paginate(context, id, window=DEFAULT_WINDOW):
  try:
    paginator = context['paginator'][id]
    page_obj = context['page_obj'][id]
    page_range = paginator.page_range
    # First and last are simply the first *n* pages and the last *n* pages,
    # where *n* is the current window size.
    first = set(page_range[:window])
    last = set(page_range[-window:])
    # Now we look around our current page, making sure that we don't wrap
    # around.
    current_start = page_obj.number-1-window
    if current_start < 0:
      current_start = 0
    current_end = page_obj.number-1+window
    if current_end < 0:
      current_end = 0
    current = set(page_range[current_start:current_end])
    pages = []
    # If there's no overlap between the first set of pages and the current
    # set of pages, then there's a possible need for elusion.
    if len(first.intersection(current)) == 0:
      first_list = list(first)
      first_list.sort()
      second_list = list(current)
      second_list.sort()
      pages.extend(first_list)
      diff = second_list[0] - first_list[-1]
      # If there is a gap of two, between the last page of the first
      # set and the first page of the current set, then we're missing a
      # page.
      if diff == 2:
        pages.append(second_list[0] - 1)
      # If the difference is just one, then there's nothing to be done,
      # as the pages need no elusion and are correct.
      elif diff == 1:
        pass
      # Otherwise, there's a bigger gap which needs to be signaled for
      # elusion, by pushing a None value to the page list.
      else:
        pages.append(None)
      pages.extend(second_list)
    else:
      unioned = list(first.union(current))
      unioned.sort()
      pages.extend(unioned)
    # If there's no overlap between the current set of pages and the last
    # set of pages, then there's a possible need for elusion.
    if len(current.intersection(last)) == 0:
      second_list = list(last)
      second_list.sort()
      diff = second_list[0] - pages[-1]
      # If there is a gap of two, between the last page of the current
      # set and the first page of the last set, then we're missing a
      # page.
      if diff == 2:
        pages.append(second_list[0] - 1)
      # If the difference is just one, then there's nothing to be done,
      # as the pages need no elusion and are correct.
      elif diff == 1:
        pass
      # Otherwise, there's a bigger gap which needs to be signaled for
      # elusion, by pushing a None value to the page list.
      else:
        pages.append(None)
      pages.extend(second_list)
    else:
      differenced = list(last.difference(current))
      differenced.sort()
      pages.extend(differenced)
    to_return = {
      'id': id, # chyba niepotrzebne
      'pages': pages,
      'page_obj': page_obj,
      'paginator': paginator,
      'is_paginated': paginator.count > paginator.per_page,
    }
    return to_return
  except KeyError, AttributeError:
      return {}

# kolejne dwa tagi wyglądają jak jakieś syfne haki...

@register.inclusion_tag('paginated_list.html', takes_context=True)
def paginated_list(context, id, page_nr=1, extra={}): # extra=HACK
  to_return = dict(extra)
  to_return.update({
    'id': id,
    'page_nr' : page_nr,
    'paginer': context['paginer'],
    'ajax': False,
    'user': context['user'],
  })
  return to_return

@register.inclusion_tag('paginated_list_rows.html', takes_context=True)
def paginated_list_ajax(context, id, page_nr=1, extra={}): # extra=HACK
  to_return = dict(extra)
  to_return.update({
    'id': id,
    'page_nr' : page_nr,
    'paginer': context['paginer'],
    'ajax': True,
    'user': context['user'],
  })
  return to_return

@register.inclusion_tag('paginated_list_panel.html', takes_context=True)
def paginated_list_panel(context, id, extra={}): # extra=HACK
  to_return = dict(extra)
  to_return.update({
    'id': id,
    'paginer': context['paginer'],
    'user': context['user'],
  })
  return to_return

@register.inclusion_tag('filter_panel.html')
def filter_panel(list_id, form):
  return {'list_id': id, 'form': form}
